/****************************************************************************
** Meta object code from reading C++ file 'kleopatraapplication.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kleopatra-26.04.2/src/kleopatraapplication.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kleopatraapplication.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN20KleopatraApplicationE_t {};
} // unnamed namespace

template <> constexpr inline auto KleopatraApplication::qt_create_metaobjectdata<qt_meta_tag_ZN20KleopatraApplicationE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KleopatraApplication",
        "setExitValue",
        "",
        "value",
        "configurationChanged",
        "distributionSettingsChanged",
        "openOrRaiseMainWindow",
        "openOrRaiseSmartCardWindow",
        "openOrRaiseConfigDialog",
        "openOrRaiseGroupsConfigDialog",
        "QWidget*",
        "parent",
        "startMonitoringSmartCard",
        "importCertificatesFromFile",
        "files",
        "GpgME::Protocol",
        "proto",
        "encryptFiles",
        "signFiles",
        "signEncryptFiles",
        "decryptFiles",
        "verifyFiles",
        "decryptVerifyFiles",
        "checksumFiles",
        "slotActivateRequested",
        "arguments",
        "workingDirectory",
        "handleFiles",
        "WId",
        "parentId",
        "blockUrl",
        "QUrl",
        "url",
        "startGpgAgent"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'setExitValue'
        QtMocHelpers::SignalData<void(int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 },
        }}),
        // Signal 'configurationChanged'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'distributionSettingsChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openOrRaiseMainWindow'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openOrRaiseSmartCardWindow'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openOrRaiseConfigDialog'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'openOrRaiseGroupsConfigDialog'
        QtMocHelpers::SlotData<void(QWidget *)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Slot 'startMonitoringSmartCard'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'importCertificatesFromFile'
        QtMocHelpers::SlotData<void(const QStringList &, GpgME::Protocol)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'encryptFiles'
        QtMocHelpers::SlotData<void(const QStringList &, GpgME::Protocol)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'signFiles'
        QtMocHelpers::SlotData<void(const QStringList &, GpgME::Protocol)>(18, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'signEncryptFiles'
        QtMocHelpers::SlotData<void(const QStringList &, GpgME::Protocol)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'decryptFiles'
        QtMocHelpers::SlotData<void(const QStringList &, GpgME::Protocol)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'verifyFiles'
        QtMocHelpers::SlotData<void(const QStringList &, GpgME::Protocol)>(21, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'decryptVerifyFiles'
        QtMocHelpers::SlotData<void(const QStringList &, GpgME::Protocol)>(22, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 }, { 0x80000000 | 15, 16 },
        }}),
        // Slot 'checksumFiles'
        QtMocHelpers::SlotData<void(const QStringList &, GpgME::Protocol)>(23, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 }, { 0x80000000 | 15, 2 },
        }}),
        // Slot 'slotActivateRequested'
        QtMocHelpers::SlotData<void(const QStringList &, const QString &)>(24, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 25 }, { QMetaType::QString, 26 },
        }}),
        // Slot 'handleFiles'
        QtMocHelpers::SlotData<void(const QStringList &, WId)>(27, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QStringList, 14 }, { 0x80000000 | 28, 29 },
        }}),
        // Slot 'handleFiles'
        QtMocHelpers::SlotData<void(const QStringList &)>(27, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::QStringList, 14 },
        }}),
        // Slot 'blockUrl'
        QtMocHelpers::SlotData<void(const QUrl &)>(30, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 31, 32 },
        }}),
        // Slot 'startGpgAgent'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KleopatraApplication, qt_meta_tag_ZN20KleopatraApplicationE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KleopatraApplication::staticMetaObject = { {
    QMetaObject::SuperData::link<QApplication::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KleopatraApplicationE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KleopatraApplicationE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN20KleopatraApplicationE_t>.metaTypes,
    nullptr
} };

void KleopatraApplication::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KleopatraApplication *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->setExitValue((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 1: _t->configurationChanged(); break;
        case 2: _t->distributionSettingsChanged(); break;
        case 3: _t->openOrRaiseMainWindow(); break;
        case 4: _t->openOrRaiseSmartCardWindow(); break;
        case 5: _t->openOrRaiseConfigDialog(); break;
        case 6: _t->openOrRaiseGroupsConfigDialog((*reinterpret_cast<std::add_pointer_t<QWidget*>>(_a[1]))); break;
        case 7: _t->startMonitoringSmartCard(); break;
        case 8: _t->importCertificatesFromFile((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GpgME::Protocol>>(_a[2]))); break;
        case 9: _t->encryptFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GpgME::Protocol>>(_a[2]))); break;
        case 10: _t->signFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GpgME::Protocol>>(_a[2]))); break;
        case 11: _t->signEncryptFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GpgME::Protocol>>(_a[2]))); break;
        case 12: _t->decryptFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GpgME::Protocol>>(_a[2]))); break;
        case 13: _t->verifyFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GpgME::Protocol>>(_a[2]))); break;
        case 14: _t->decryptVerifyFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GpgME::Protocol>>(_a[2]))); break;
        case 15: _t->checksumFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<GpgME::Protocol>>(_a[2]))); break;
        case 16: _t->slotActivateRequested((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QString>>(_a[2]))); break;
        case 17: _t->handleFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<WId>>(_a[2]))); break;
        case 18: _t->handleFiles((*reinterpret_cast<std::add_pointer_t<QStringList>>(_a[1]))); break;
        case 19: _t->blockUrl((*reinterpret_cast<std::add_pointer_t<QUrl>>(_a[1]))); break;
        case 20: _t->startGpgAgent(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KleopatraApplication::*)(int )>(_a, &KleopatraApplication::setExitValue, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleopatraApplication::*)()>(_a, &KleopatraApplication::configurationChanged, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (KleopatraApplication::*)()>(_a, &KleopatraApplication::distributionSettingsChanged, 2))
            return;
    }
}

const QMetaObject *KleopatraApplication::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KleopatraApplication::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN20KleopatraApplicationE_t>.strings))
        return static_cast<void*>(this);
    return QApplication::qt_metacast(_clname);
}

int KleopatraApplication::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QApplication::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void KleopatraApplication::setExitValue(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void KleopatraApplication::configurationChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void KleopatraApplication::distributionSettingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
