/****************************************************************************
** Meta object code from reading C++ file 'readerstatus.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kleopatra-26.04.2/src/smartcard/readerstatus.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'readerstatus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4Kleo9SmartCard12ReaderStatusE_t {};
} // unnamed namespace

template <> constexpr inline auto Kleo::SmartCard::ReaderStatus::qt_create_metaobjectdata<qt_meta_tag_ZN4Kleo9SmartCard12ReaderStatusE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kleo::SmartCard::ReaderStatus",
        "firstCardWithNullPinChanged",
        "",
        "std::string",
        "serialNumber",
        "cardAdded",
        "appName",
        "cardChanged",
        "cardRemoved",
        "currentActionChanged",
        "Action",
        "action",
        "updateCardsStarted",
        "updateCardStarted",
        "updateFinished",
        "startingLearnCard",
        "cardLearned",
        "startOfGpgAgentRequested",
        "updateStatus",
        "updateCard",
        "learnCard",
        "startMonitoring"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'firstCardWithNullPinChanged'
        QtMocHelpers::SignalData<void(const std::string &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'cardAdded'
        QtMocHelpers::SignalData<void(const std::string &, const std::string &)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 6 },
        }}),
        // Signal 'cardChanged'
        QtMocHelpers::SignalData<void(const std::string &, const std::string &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 6 },
        }}),
        // Signal 'cardRemoved'
        QtMocHelpers::SignalData<void(const std::string &, const std::string &)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 6 },
        }}),
        // Signal 'currentActionChanged'
        QtMocHelpers::SignalData<void(enum Action)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 11 },
        }}),
        // Signal 'updateCardsStarted'
        QtMocHelpers::SignalData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'updateCardStarted'
        QtMocHelpers::SignalData<void(const std::string &, const std::string &)>(13, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 6 },
        }}),
        // Signal 'updateFinished'
        QtMocHelpers::SignalData<void()>(14, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'startingLearnCard'
        QtMocHelpers::SignalData<void(const std::string &, const std::string &)>(15, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 6 },
        }}),
        // Signal 'cardLearned'
        QtMocHelpers::SignalData<void(const std::string &, const std::string &)>(16, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 6 },
        }}),
        // Signal 'startOfGpgAgentRequested'
        QtMocHelpers::SignalData<void()>(17, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateStatus'
        QtMocHelpers::SlotData<void()>(18, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateCard'
        QtMocHelpers::SlotData<void(const std::string &, const std::string &)>(19, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 6 },
        }}),
        // Slot 'learnCard'
        QtMocHelpers::SlotData<void(const std::string &, const std::string &)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 }, { 0x80000000 | 3, 6 },
        }}),
        // Slot 'startMonitoring'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<ReaderStatus, qt_meta_tag_ZN4Kleo9SmartCard12ReaderStatusE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kleo::SmartCard::ReaderStatus::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Kleo9SmartCard12ReaderStatusE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Kleo9SmartCard12ReaderStatusE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4Kleo9SmartCard12ReaderStatusE_t>.metaTypes,
    nullptr
} };

void Kleo::SmartCard::ReaderStatus::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<ReaderStatus *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->firstCardWithNullPinChanged((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1]))); break;
        case 1: _t->cardAdded((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 2: _t->cardChanged((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 3: _t->cardRemoved((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 4: _t->currentActionChanged((*reinterpret_cast<std::add_pointer_t<enum Action>>(_a[1]))); break;
        case 5: _t->updateCardsStarted(); break;
        case 6: _t->updateCardStarted((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 7: _t->updateFinished(); break;
        case 8: _t->startingLearnCard((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 9: _t->cardLearned((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 10: _t->startOfGpgAgentRequested(); break;
        case 11: _t->updateStatus(); break;
        case 12: _t->updateCard((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 13: _t->learnCard((*reinterpret_cast<std::add_pointer_t<std::string>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<std::string>>(_a[2]))); break;
        case 14: _t->startMonitoring(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)(const std::string & )>(_a, &ReaderStatus::firstCardWithNullPinChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)(const std::string & , const std::string & )>(_a, &ReaderStatus::cardAdded, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)(const std::string & , const std::string & )>(_a, &ReaderStatus::cardChanged, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)(const std::string & , const std::string & )>(_a, &ReaderStatus::cardRemoved, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)(Action )>(_a, &ReaderStatus::currentActionChanged, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)()>(_a, &ReaderStatus::updateCardsStarted, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)(const std::string & , const std::string & )>(_a, &ReaderStatus::updateCardStarted, 6))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)()>(_a, &ReaderStatus::updateFinished, 7))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)(const std::string & , const std::string & )>(_a, &ReaderStatus::startingLearnCard, 8))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)(const std::string & , const std::string & )>(_a, &ReaderStatus::cardLearned, 9))
            return;
        if (QtMocHelpers::indexOfMethod<void (ReaderStatus::*)()>(_a, &ReaderStatus::startOfGpgAgentRequested, 10))
            return;
    }
}

const QMetaObject *Kleo::SmartCard::ReaderStatus::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kleo::SmartCard::ReaderStatus::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Kleo9SmartCard12ReaderStatusE_t>.strings))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int Kleo::SmartCard::ReaderStatus::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 15)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 15;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 15)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 15;
    }
    return _id;
}

// SIGNAL 0
void Kleo::SmartCard::ReaderStatus::firstCardWithNullPinChanged(const std::string & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Kleo::SmartCard::ReaderStatus::cardAdded(const std::string & _t1, const std::string & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2);
}

// SIGNAL 2
void Kleo::SmartCard::ReaderStatus::cardChanged(const std::string & _t1, const std::string & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2);
}

// SIGNAL 3
void Kleo::SmartCard::ReaderStatus::cardRemoved(const std::string & _t1, const std::string & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1, _t2);
}

// SIGNAL 4
void Kleo::SmartCard::ReaderStatus::currentActionChanged(Action _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void Kleo::SmartCard::ReaderStatus::updateCardsStarted()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void Kleo::SmartCard::ReaderStatus::updateCardStarted(const std::string & _t1, const std::string & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 6, nullptr, _t1, _t2);
}

// SIGNAL 7
void Kleo::SmartCard::ReaderStatus::updateFinished()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void Kleo::SmartCard::ReaderStatus::startingLearnCard(const std::string & _t1, const std::string & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 8, nullptr, _t1, _t2);
}

// SIGNAL 9
void Kleo::SmartCard::ReaderStatus::cardLearned(const std::string & _t1, const std::string & _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 9, nullptr, _t1, _t2);
}

// SIGNAL 10
void Kleo::SmartCard::ReaderStatus::startOfGpgAgentRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 10, nullptr);
}
QT_WARNING_POP
