/********************************************************************************
** Form generated from reading UI file 'setinitialpindialog.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SETINITIALPINDIALOG_H
#define UI_SETINITIALPINDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QAbstractButton>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SetInitialPinDialog
{
public:
    QGridLayout *gridLayout;
    QLabel *mainLB;
    QFrame *line_3;
    QLabel *nksLB;
    QPushButton *nksPB;
    QLabel *nksResultIcon;
    QLabel *nksStatusLB;
    QFrame *line;
    QLabel *sigGLB;
    QPushButton *sigGPB;
    QLabel *sigGResultIcon;
    QLabel *sigGStatusLB;
    QFrame *line_2;
    QSpacerItem *spacer;
    QDialogButtonBox *dialogButtonBox;

    void setupUi(QDialog *SetInitialPinDialog)
    {
        if (SetInitialPinDialog->objectName().isEmpty())
            SetInitialPinDialog->setObjectName("SetInitialPinDialog");
        gridLayout = new QGridLayout(SetInitialPinDialog);
        gridLayout->setObjectName("gridLayout");
        mainLB = new QLabel(SetInitialPinDialog);
        mainLB->setObjectName("mainLB");

        gridLayout->addWidget(mainLB, 0, 0, 1, 3);

        line_3 = new QFrame(SetInitialPinDialog);
        line_3->setObjectName("line_3");
        line_3->setFrameShape(QFrame::Shape::HLine);
        line_3->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line_3, 1, 0, 1, 3);

        nksLB = new QLabel(SetInitialPinDialog);
        nksLB->setObjectName("nksLB");

        gridLayout->addWidget(nksLB, 2, 0, 1, 2);

        nksPB = new QPushButton(SetInitialPinDialog);
        nksPB->setObjectName("nksPB");

        gridLayout->addWidget(nksPB, 2, 2, 1, 1);

        nksResultIcon = new QLabel(SetInitialPinDialog);
        nksResultIcon->setObjectName("nksResultIcon");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(nksResultIcon->sizePolicy().hasHeightForWidth());
        nksResultIcon->setSizePolicy(sizePolicy);
        nksResultIcon->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(nksResultIcon, 3, 0, 1, 1);

        nksStatusLB = new QLabel(SetInitialPinDialog);
        nksStatusLB->setObjectName("nksStatusLB");

        gridLayout->addWidget(nksStatusLB, 3, 1, 1, 2);

        line = new QFrame(SetInitialPinDialog);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line, 4, 0, 1, 3);

        sigGLB = new QLabel(SetInitialPinDialog);
        sigGLB->setObjectName("sigGLB");
        sigGLB->setEnabled(false);

        gridLayout->addWidget(sigGLB, 5, 0, 1, 2);

        sigGPB = new QPushButton(SetInitialPinDialog);
        sigGPB->setObjectName("sigGPB");
        sigGPB->setEnabled(false);

        gridLayout->addWidget(sigGPB, 5, 2, 1, 1);

        sigGResultIcon = new QLabel(SetInitialPinDialog);
        sigGResultIcon->setObjectName("sigGResultIcon");
        sizePolicy.setHeightForWidth(sigGResultIcon->sizePolicy().hasHeightForWidth());
        sigGResultIcon->setSizePolicy(sizePolicy);
        sigGResultIcon->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        gridLayout->addWidget(sigGResultIcon, 6, 0, 1, 1);

        sigGStatusLB = new QLabel(SetInitialPinDialog);
        sigGStatusLB->setObjectName("sigGStatusLB");

        gridLayout->addWidget(sigGStatusLB, 6, 1, 1, 2);

        line_2 = new QFrame(SetInitialPinDialog);
        line_2->setObjectName("line_2");
        line_2->setFrameShape(QFrame::Shape::HLine);
        line_2->setFrameShadow(QFrame::Shadow::Sunken);

        gridLayout->addWidget(line_2, 7, 0, 1, 3);

        spacer = new QSpacerItem(13, 13, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacer, 8, 0, 1, 1);

        dialogButtonBox = new QDialogButtonBox(SetInitialPinDialog);
        dialogButtonBox->setObjectName("dialogButtonBox");
        dialogButtonBox->setOrientation(Qt::Horizontal);
        dialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Close);

        gridLayout->addWidget(dialogButtonBox, 9, 0, 1, 3);


        retranslateUi(SetInitialPinDialog);
        QObject::connect(dialogButtonBox, &QDialogButtonBox::rejected, SetInitialPinDialog, qOverload<>(&QDialog::reject));
        QObject::connect(nksPB, SIGNAL(clicked()), SetInitialPinDialog, SLOT(slotNksButtonClicked()));
        QObject::connect(sigGPB, SIGNAL(clicked()), SetInitialPinDialog, SLOT(slotSigGButtonClicked()));

        QMetaObject::connectSlotsByName(SetInitialPinDialog);
    } // setupUi

    void retranslateUi(QDialog *SetInitialPinDialog)
    {
        mainLB->setText(tr2i18n("<p>On this SmartCard, there is space for two certificates:\n"
"<ol>\n"
"<li>A normal certificate</li>\n"
"<li>A special certificate for making <em>qualified signatures</em> according to the German Signaturgesetz</li>\n"
"</ol>\n"
"You need to set initial PINs before you can use them.</p>", nullptr));
        nksLB->setText(tr2i18n("Step 1: Set the initial PIN for the first certificate (\"NKS\"):", nullptr));
        nksPB->setText(tr2i18n("Set Initial PIN (NKS)", nullptr));
        nksStatusLB->setText(QString());
        sigGLB->setText(tr2i18n("Step 2: Set the initial PIN for the qualified signature certificate (\"SigG\"):", nullptr));
        sigGPB->setText(tr2i18n("(optional) Set Initial PIN (SigG)", nullptr));
        sigGStatusLB->setText(QString());
        (void)SetInitialPinDialog;
    } // retranslateUi

};

namespace Ui {
    class SetInitialPinDialog: public Ui_SetInitialPinDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SETINITIALPINDIALOG_H

