/****************************************************************************
** Meta object code from reading C++ file 'lookupcertificatesdialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kleopatra-26.04.2/src/dialogs/lookupcertificatesdialog.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'lookupcertificatesdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4Kleo7Dialogs24LookupCertificatesDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto Kleo::Dialogs::LookupCertificatesDialog::qt_create_metaobjectdata<qt_meta_tag_ZN4Kleo7Dialogs24LookupCertificatesDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kleo::Dialogs::LookupCertificatesDialog",
        "searchTextChanged",
        "",
        "text",
        "saveAsRequested",
        "std::vector<GpgME::Key>",
        "certs",
        "importRequested",
        "std::vector<KeyWithOrigin>",
        "detailsRequested",
        "GpgME::Key",
        "accept",
        "slotSearchTextChanged",
        "slotSearchClicked",
        "slotSelectionChanged",
        "slotDetailsClicked",
        "slotSaveAsClicked"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'searchTextChanged'
        QtMocHelpers::SignalData<void(const QString &)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::QString, 3 },
        }}),
        // Signal 'saveAsRequested'
        QtMocHelpers::SignalData<void(const std::vector<GpgME::Key> &)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 5, 6 },
        }}),
        // Signal 'importRequested'
        QtMocHelpers::SignalData<void(const std::vector<KeyWithOrigin> &)>(7, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 8, 6 },
        }}),
        // Signal 'detailsRequested'
        QtMocHelpers::SignalData<void(const GpgME::Key &)>(9, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 10, 6 },
        }}),
        // Slot 'accept'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'slotSearchTextChanged'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSearchClicked'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSelectionChanged'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotDetailsClicked'
        QtMocHelpers::SlotData<void()>(15, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'slotSaveAsClicked'
        QtMocHelpers::SlotData<void()>(16, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<LookupCertificatesDialog, qt_meta_tag_ZN4Kleo7Dialogs24LookupCertificatesDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kleo::Dialogs::LookupCertificatesDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Kleo7Dialogs24LookupCertificatesDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Kleo7Dialogs24LookupCertificatesDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4Kleo7Dialogs24LookupCertificatesDialogE_t>.metaTypes,
    nullptr
} };

void Kleo::Dialogs::LookupCertificatesDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<LookupCertificatesDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->searchTextChanged((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 1: _t->saveAsRequested((*reinterpret_cast<std::add_pointer_t<std::vector<GpgME::Key>>>(_a[1]))); break;
        case 2: _t->importRequested((*reinterpret_cast<std::add_pointer_t<std::vector<KeyWithOrigin>>>(_a[1]))); break;
        case 3: _t->detailsRequested((*reinterpret_cast<std::add_pointer_t<GpgME::Key>>(_a[1]))); break;
        case 4: _t->accept(); break;
        case 5: _t->d->slotSearchTextChanged(); break;
        case 6: _t->d->slotSearchClicked(); break;
        case 7: _t->d->slotSelectionChanged(); break;
        case 8: _t->d->slotDetailsClicked(); break;
        case 9: _t->d->slotSaveAsClicked(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (LookupCertificatesDialog::*)(const QString & )>(_a, &LookupCertificatesDialog::searchTextChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (LookupCertificatesDialog::*)(const std::vector<GpgME::Key> & )>(_a, &LookupCertificatesDialog::saveAsRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (LookupCertificatesDialog::*)(const std::vector<KeyWithOrigin> & )>(_a, &LookupCertificatesDialog::importRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (LookupCertificatesDialog::*)(const GpgME::Key & )>(_a, &LookupCertificatesDialog::detailsRequested, 3))
            return;
    }
}

const QMetaObject *Kleo::Dialogs::LookupCertificatesDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kleo::Dialogs::LookupCertificatesDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Kleo7Dialogs24LookupCertificatesDialogE_t>.strings))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int Kleo::Dialogs::LookupCertificatesDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType();
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void Kleo::Dialogs::LookupCertificatesDialog::searchTextChanged(const QString & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Kleo::Dialogs::LookupCertificatesDialog::saveAsRequested(const std::vector<GpgME::Key> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}

// SIGNAL 2
void Kleo::Dialogs::LookupCertificatesDialog::importRequested(const std::vector<KeyWithOrigin> & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1);
}

// SIGNAL 3
void Kleo::Dialogs::LookupCertificatesDialog::detailsRequested(const GpgME::Key & _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 3, nullptr, _t1);
}
QT_WARNING_POP
