/********************************************************************************
** Form generated from reading UI file 'smimevalidationconfigurationwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SMIMEVALIDATIONCONFIGURATIONWIDGET_H
#define UI_SMIMEVALIDATIONCONFIGURATIONWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "libkleo/keyrequester.h"

namespace Kleo {
namespace Config {

class Ui_SMimeValidationConfigurationWidget
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QCheckBox *intervalRefreshCB;
    QSpinBox *intervalRefreshSB;
    QSpacerItem *horizontalSpacer;
    QCheckBox *OCSPCB;
    QGroupBox *OCSPGroupBox;
    QGridLayout *gridLayout;
    QLabel *OCSPResponderURLLabel;
    QLineEdit *OCSPResponderURL;
    QLabel *OCSPResponderSignatureLabel;
    QCheckBox *ignoreServiceURLCB;
    QHBoxLayout *hboxLayout;
    QLabel *OCSPResponderSignature;
    QPushButton *ocspSelectButton;
    QCheckBox *doNotCheckCertPolicyCB;
    QCheckBox *neverConsultCB;
    QCheckBox *fetchMissingCB;
    QTabWidget *requestsTabs;
    QWidget *tabHTTP;
    QGridLayout *gridLayout1;
    QCheckBox *disableHTTPCB;
    QCheckBox *ignoreHTTPDPCB;
    QRadioButton *honorHTTPProxyRB;
    QLabel *systemHTTPProxy;
    QRadioButton *useCustomHTTPProxyRB;
    QLineEdit *customHTTPProxy;
    QSpacerItem *spacerItem;
    QWidget *tabLDAP;
    QGridLayout *gridLayout2;
    QCheckBox *disableLDAPCB;
    QCheckBox *ignoreLDAPDPCB;
    QLabel *customLDAPLabel;
    QLineEdit *customLDAPProxy;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *Kleo__Config__SMimeValidationConfigurationWidget)
    {
        if (Kleo__Config__SMimeValidationConfigurationWidget->objectName().isEmpty())
            Kleo__Config__SMimeValidationConfigurationWidget->setObjectName("Kleo__Config__SMimeValidationConfigurationWidget");
        Kleo__Config__SMimeValidationConfigurationWidget->resize(502, 603);
        verticalLayout = new QVBoxLayout(Kleo__Config__SMimeValidationConfigurationWidget);
        verticalLayout->setObjectName("verticalLayout");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        intervalRefreshCB = new QCheckBox(Kleo__Config__SMimeValidationConfigurationWidget);
        intervalRefreshCB->setObjectName("intervalRefreshCB");

        horizontalLayout->addWidget(intervalRefreshCB);

        intervalRefreshSB = new QSpinBox(Kleo__Config__SMimeValidationConfigurationWidget);
        intervalRefreshSB->setObjectName("intervalRefreshSB");
        intervalRefreshSB->setEnabled(false);
        intervalRefreshSB->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        intervalRefreshSB->setMinimum(1);
        intervalRefreshSB->setMaximum(24);

        horizontalLayout->addWidget(intervalRefreshSB);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        OCSPCB = new QCheckBox(Kleo__Config__SMimeValidationConfigurationWidget);
        OCSPCB->setObjectName("OCSPCB");

        verticalLayout->addWidget(OCSPCB);

        OCSPGroupBox = new QGroupBox(Kleo__Config__SMimeValidationConfigurationWidget);
        OCSPGroupBox->setObjectName("OCSPGroupBox");
        OCSPGroupBox->setEnabled(false);
        gridLayout = new QGridLayout(OCSPGroupBox);
        gridLayout->setObjectName("gridLayout");
        OCSPResponderURLLabel = new QLabel(OCSPGroupBox);
        OCSPResponderURLLabel->setObjectName("OCSPResponderURLLabel");
        OCSPResponderURLLabel->setWordWrap(false);

        gridLayout->addWidget(OCSPResponderURLLabel, 0, 0, 1, 1);

        OCSPResponderURL = new QLineEdit(OCSPGroupBox);
        OCSPResponderURL->setObjectName("OCSPResponderURL");

        gridLayout->addWidget(OCSPResponderURL, 0, 1, 1, 1);

        OCSPResponderSignatureLabel = new QLabel(OCSPGroupBox);
        OCSPResponderSignatureLabel->setObjectName("OCSPResponderSignatureLabel");
        OCSPResponderSignatureLabel->setWordWrap(false);

        gridLayout->addWidget(OCSPResponderSignatureLabel, 1, 0, 1, 1);

        ignoreServiceURLCB = new QCheckBox(OCSPGroupBox);
        ignoreServiceURLCB->setObjectName("ignoreServiceURLCB");

        gridLayout->addWidget(ignoreServiceURLCB, 2, 0, 1, 2);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName("hboxLayout");
        OCSPResponderSignature = new QLabel(OCSPGroupBox);
        OCSPResponderSignature->setObjectName("OCSPResponderSignature");

        hboxLayout->addWidget(OCSPResponderSignature);

        ocspSelectButton = new QPushButton(OCSPGroupBox);
        ocspSelectButton->setObjectName("ocspSelectButton");

        hboxLayout->addWidget(ocspSelectButton);


        gridLayout->addLayout(hboxLayout, 1, 1, 1, 1);


        verticalLayout->addWidget(OCSPGroupBox);

        doNotCheckCertPolicyCB = new QCheckBox(Kleo__Config__SMimeValidationConfigurationWidget);
        doNotCheckCertPolicyCB->setObjectName("doNotCheckCertPolicyCB");

        verticalLayout->addWidget(doNotCheckCertPolicyCB);

        neverConsultCB = new QCheckBox(Kleo__Config__SMimeValidationConfigurationWidget);
        neverConsultCB->setObjectName("neverConsultCB");

        verticalLayout->addWidget(neverConsultCB);

        fetchMissingCB = new QCheckBox(Kleo__Config__SMimeValidationConfigurationWidget);
        fetchMissingCB->setObjectName("fetchMissingCB");

        verticalLayout->addWidget(fetchMissingCB);

        requestsTabs = new QTabWidget(Kleo__Config__SMimeValidationConfigurationWidget);
        requestsTabs->setObjectName("requestsTabs");
        tabHTTP = new QWidget();
        tabHTTP->setObjectName("tabHTTP");
        gridLayout1 = new QGridLayout(tabHTTP);
        gridLayout1->setObjectName("gridLayout1");
        disableHTTPCB = new QCheckBox(tabHTTP);
        disableHTTPCB->setObjectName("disableHTTPCB");

        gridLayout1->addWidget(disableHTTPCB, 0, 0, 1, 2);

        ignoreHTTPDPCB = new QCheckBox(tabHTTP);
        ignoreHTTPDPCB->setObjectName("ignoreHTTPDPCB");

        gridLayout1->addWidget(ignoreHTTPDPCB, 1, 0, 1, 2);

        honorHTTPProxyRB = new QRadioButton(tabHTTP);
        honorHTTPProxyRB->setObjectName("honorHTTPProxyRB");

        gridLayout1->addWidget(honorHTTPProxyRB, 2, 0, 1, 1);

        systemHTTPProxy = new QLabel(tabHTTP);
        systemHTTPProxy->setObjectName("systemHTTPProxy");
        systemHTTPProxy->setWordWrap(false);

        gridLayout1->addWidget(systemHTTPProxy, 2, 1, 1, 1);

        useCustomHTTPProxyRB = new QRadioButton(tabHTTP);
        useCustomHTTPProxyRB->setObjectName("useCustomHTTPProxyRB");

        gridLayout1->addWidget(useCustomHTTPProxyRB, 3, 0, 1, 1);

        customHTTPProxy = new QLineEdit(tabHTTP);
        customHTTPProxy->setObjectName("customHTTPProxy");

        gridLayout1->addWidget(customHTTPProxy, 3, 1, 1, 1);

        spacerItem = new QSpacerItem(320, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout1->addItem(spacerItem, 4, 0, 1, 2);

        requestsTabs->addTab(tabHTTP, QString());
        tabLDAP = new QWidget();
        tabLDAP->setObjectName("tabLDAP");
        gridLayout2 = new QGridLayout(tabLDAP);
        gridLayout2->setObjectName("gridLayout2");
        disableLDAPCB = new QCheckBox(tabLDAP);
        disableLDAPCB->setObjectName("disableLDAPCB");

        gridLayout2->addWidget(disableLDAPCB, 0, 0, 1, 2);

        ignoreLDAPDPCB = new QCheckBox(tabLDAP);
        ignoreLDAPDPCB->setObjectName("ignoreLDAPDPCB");

        gridLayout2->addWidget(ignoreLDAPDPCB, 1, 0, 1, 2);

        customLDAPLabel = new QLabel(tabLDAP);
        customLDAPLabel->setObjectName("customLDAPLabel");
        customLDAPLabel->setWordWrap(false);

        gridLayout2->addWidget(customLDAPLabel, 2, 0, 1, 1);

        customLDAPProxy = new QLineEdit(tabLDAP);
        customLDAPProxy->setObjectName("customLDAPProxy");

        gridLayout2->addWidget(customLDAPProxy, 2, 1, 1, 1);

        spacerItem1 = new QSpacerItem(320, 16, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout2->addItem(spacerItem1, 3, 0, 1, 2);

        requestsTabs->addTab(tabLDAP, QString());

        verticalLayout->addWidget(requestsTabs);


        retranslateUi(Kleo__Config__SMimeValidationConfigurationWidget);
        QObject::connect(OCSPCB, &QCheckBox::toggled, OCSPGroupBox, &QGroupBox::setEnabled);
        QObject::connect(useCustomHTTPProxyRB, &QRadioButton::toggled, customHTTPProxy, &QLineEdit::setEnabled);
        QObject::connect(disableLDAPCB, &QCheckBox::toggled, ignoreLDAPDPCB, &QCheckBox::setDisabled);
        QObject::connect(disableLDAPCB, &QCheckBox::toggled, customLDAPLabel, &QLabel::setDisabled);
        QObject::connect(disableLDAPCB, &QCheckBox::toggled, customLDAPProxy, &QLineEdit::setDisabled);
        QObject::connect(disableHTTPCB, &QCheckBox::toggled, ignoreHTTPDPCB, &QCheckBox::setDisabled);
        QObject::connect(disableHTTPCB, &QCheckBox::toggled, honorHTTPProxyRB, &QRadioButton::setDisabled);
        QObject::connect(disableHTTPCB, &QCheckBox::toggled, useCustomHTTPProxyRB, &QRadioButton::setDisabled);
        QObject::connect(intervalRefreshCB, &QCheckBox::toggled, intervalRefreshSB, &QSpinBox::setEnabled);

        QMetaObject::connectSlotsByName(Kleo__Config__SMimeValidationConfigurationWidget);
    } // setupUi

    void retranslateUi(QWidget *Kleo__Config__SMimeValidationConfigurationWidget)
    {
#if QT_CONFIG(whatsthis)
        intervalRefreshCB->setWhatsThis(tr2i18n("This option enables interval checking of certificate validity. You can also choose the checking interval (in hours). Note that validation is performed implicitly whenever significant files in ~/.gnupg change. This option therefore only affects external factors of certificate validity.", nullptr));
#endif // QT_CONFIG(whatsthis)
        intervalRefreshCB->setText(tr2i18n("Check certificate validity every", nullptr));
#if QT_CONFIG(whatsthis)
        intervalRefreshSB->setWhatsThis(tr2i18n("This option enables interval checking of certificate validity. You can also choose the checking interval (in hours). Note that validation is performed implicitly whenever significant files in ~/.gnupg change. This option therefore only affects external factors of certificate validity.", nullptr));
#endif // QT_CONFIG(whatsthis)
        intervalRefreshSB->setSpecialValueText(tr2i18n("hour", nullptr));
        intervalRefreshSB->setSuffix(tr2i18n(" hours", nullptr));
#if QT_CONFIG(whatsthis)
        OCSPCB->setWhatsThis(tr2i18n("If this option is selected, S/MIME certificates are validated online using the Online Certificates Status Protocol (OCSP). Fill in the URL of the OCSP responder below.", nullptr));
#endif // QT_CONFIG(whatsthis)
        OCSPCB->setText(tr2i18n("Validate certificates online (OCSP)", nullptr));
        OCSPGroupBox->setTitle(tr2i18n("Online Certificate Validation", nullptr));
        OCSPResponderURLLabel->setText(tr2i18n("OCSP responder URL:", nullptr));
#if QT_CONFIG(whatsthis)
        OCSPResponderURL->setWhatsThis(tr2i18n("Enter here the address of the server for online validation of certificates (OCSP responder). The URL is usually starting with http://.", nullptr));
#endif // QT_CONFIG(whatsthis)
        OCSPResponderSignatureLabel->setText(tr2i18n("OCSP responder signature:", nullptr));
        ignoreServiceURLCB->setText(tr2i18n("Ignore service URL of certificates", nullptr));
#if QT_CONFIG(whatsthis)
        OCSPResponderSignature->setWhatsThis(tr2i18n("Choose here the certificate with which the OCSP server signs its replies.", nullptr));
#endif // QT_CONFIG(whatsthis)
        ocspSelectButton->setText(tr2i18n("Change Certificate\342\200\246", nullptr));
#if QT_CONFIG(whatsthis)
        doNotCheckCertPolicyCB->setWhatsThis(tr2i18n("By default, GnuPG uses the file ~/.gnupg/policies.txt to check if a certificate policy is allowed. If this option is selected, policies are not checked.", nullptr));
#endif // QT_CONFIG(whatsthis)
        doNotCheckCertPolicyCB->setText(tr2i18n("Do not check certificate policies", nullptr));
#if QT_CONFIG(whatsthis)
        neverConsultCB->setWhatsThis(tr2i18n("If this option is checked, Certificate Revocation Lists are never used to validate S/MIME certificates.", nullptr));
#endif // QT_CONFIG(whatsthis)
        neverConsultCB->setText(tr2i18n("Never consult a CRL", nullptr));
#if QT_CONFIG(whatsthis)
        fetchMissingCB->setWhatsThis(tr2i18n("If this option is checked, missing issuer certificates are fetched when necessary (this applies to both validation methods, CRLs and OCSP).", nullptr));
#endif // QT_CONFIG(whatsthis)
        fetchMissingCB->setText(tr2i18n("Fetch missing issuer certificates", nullptr));
#if QT_CONFIG(whatsthis)
        disableHTTPCB->setWhatsThis(tr2i18n("Entirely disables the use of HTTP for S/MIME.", nullptr));
#endif // QT_CONFIG(whatsthis)
        disableHTTPCB->setText(tr2i18n("Do not perform any HTTP requests", nullptr));
#if QT_CONFIG(whatsthis)
        ignoreHTTPDPCB->setWhatsThis(tr2i18n("When looking for the location of a CRL, the to-be-tested certificate usually contains what are known as \"CRL Distribution Point\" (DP) entries, which are URLs describing the way to access the CRL. The first-found DP entry is used.  With this option, all entries using the HTTP scheme are ignored when looking for a suitable DP.", nullptr));
#endif // QT_CONFIG(whatsthis)
        ignoreHTTPDPCB->setText(tr2i18n("Ignore HTTP CRL distribution point of certificates", nullptr));
#if QT_CONFIG(whatsthis)
        honorHTTPProxyRB->setWhatsThis(tr2i18n("If this option is selected, the value of the HTTP proxy shown on the right (which comes from the environment variable http_proxy) will be used for any HTTP request.", nullptr));
#endif // QT_CONFIG(whatsthis)
        honorHTTPProxyRB->setText(tr2i18n("Use system HTTP proxy:", nullptr));
        systemHTTPProxy->setText(tr2i18n("no proxy", nullptr));
        useCustomHTTPProxyRB->setText(tr2i18n("Use this proxy for HTTP requests: ", nullptr));
#if QT_CONFIG(whatsthis)
        customHTTPProxy->setWhatsThis(tr2i18n("<p>If no system proxy is set, or you need to use a different proxy for GpgSM, you can enter its location here.</p><p>It will be used for all HTTP requests relating to S/MIME.</p><p>The syntax is host:port, for instance myproxy.nowhere.com:3128.</p>", nullptr));
#endif // QT_CONFIG(whatsthis)
        requestsTabs->setTabText(requestsTabs->indexOf(tabHTTP), tr2i18n("&HTTP Requests", nullptr));
#if QT_CONFIG(whatsthis)
        disableLDAPCB->setWhatsThis(tr2i18n("Entirely disables the use of LDAP for S/MIME.", nullptr));
#endif // QT_CONFIG(whatsthis)
        disableLDAPCB->setText(tr2i18n("Do not perform any LDAP requests", nullptr));
#if QT_CONFIG(whatsthis)
        ignoreLDAPDPCB->setWhatsThis(tr2i18n("When looking for the location of a CRL, the to-be-tested certificate usually contains what are known as \"CRL Distribution Point\" (DP) entries, which are URLs describing the way to access the CRL. The first found DP entry is used.  With this option, all entries using the LDAP scheme are ignored when looking for a suitable DP.", nullptr));
#endif // QT_CONFIG(whatsthis)
        ignoreLDAPDPCB->setText(tr2i18n("Ignore LDAP CRL distribution point of certificates", nullptr));
        customLDAPLabel->setText(tr2i18n("Primary host for LDAP requests:", nullptr));
#if QT_CONFIG(whatsthis)
        customLDAPProxy->setWhatsThis(tr2i18n("Entering an LDAP server here will make all LDAP requests go to that server first. More precisely, this setting overrides any specified host and port part in a LDAP URL and will also be used if host and port have been omitted from the URL. Other LDAP servers will be used only if the connection to the \"proxy\" failed.\n"
"The syntax is \"HOST\" or \"HOST:PORT\". If PORT is omitted, port 389 (standard LDAP port) is used.", nullptr));
#endif // QT_CONFIG(whatsthis)
        requestsTabs->setTabText(requestsTabs->indexOf(tabLDAP), tr2i18n("&LDAP Requests", nullptr));
        (void)Kleo__Config__SMimeValidationConfigurationWidget;
    } // retranslateUi

};

} // namespace Config
} // namespace Kleo

namespace Kleo {
namespace Config {
namespace Ui {
    class SMimeValidationConfigurationWidget: public Ui_SMimeValidationConfigurationWidget {};
} // namespace Ui
} // namespace Config
} // namespace Kleo

#endif // SMIMEVALIDATIONCONFIGURATIONWIDGET_H

