// This file is generated by kconfig_compiler_kf6 from settings.kcfg.
// All changes you do to this file will be lost.

#include "settings.h"

using namespace Kleo;

Settings::Settings( )
  : KConfigSkeleton()
{
  setCurrentGroup( QStringLiteral( "CertificateCreationWizard" ) );

  KConfigSkeleton::ItemString  *itemCnPlaceholder;
  itemCnPlaceholder = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "CN_placeholder" ), mCnPlaceholder );
  addItem( itemCnPlaceholder, QStringLiteral( "cnPlaceholder" ) );
  KConfigSkeleton::ItemBool  *itemPrefillCN;
  itemPrefillCN = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "CN_prefill" ), mPrefillCN, true );
  addItem( itemPrefillCN, QStringLiteral( "prefillCN" ) );
  KConfigSkeleton::ItemString  *itemEmailPlaceholder;
  itemEmailPlaceholder = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "EMAIL_placeholder" ), mEmailPlaceholder );
  addItem( itemEmailPlaceholder, QStringLiteral( "emailPlaceholder" ) );
  KConfigSkeleton::ItemBool  *itemPrefillEmail;
  itemPrefillEmail = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "EMAIL_prefill" ), mPrefillEmail, true );
  addItem( itemPrefillEmail, QStringLiteral( "prefillEmail" ) );
  KConfigSkeleton::ItemString  *itemEmail;
  itemEmail = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "EMAIL" ), mEmail );
  addItem( itemEmail, QStringLiteral( "email" ) );
  KConfigSkeleton::ItemString  *itemNamePlaceholder;
  itemNamePlaceholder = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "NAME_placeholder" ), mNamePlaceholder );
  addItem( itemNamePlaceholder, QStringLiteral( "namePlaceholder" ) );
  KConfigSkeleton::ItemBool  *itemPrefillName;
  itemPrefillName = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "NAME_prefill" ), mPrefillName, true );
  addItem( itemPrefillName, QStringLiteral( "prefillName" ) );
  KConfigSkeleton::ItemString  *itemName;
  itemName = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "NAME" ), mName );
  addItem( itemName, QStringLiteral( "name" ) );
  KConfigSkeleton::ItemInt  *itemValidityPeriodInDays;
  itemValidityPeriodInDays = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "ValidityPeriodInDays" ), mValidityPeriodInDays, -1 );
  addItem( itemValidityPeriodInDays, QStringLiteral( "ValidityPeriodInDays" ) );
  KConfigSkeleton::ItemInt  *itemValidityPeriodInDaysMin;
  itemValidityPeriodInDaysMin = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "ValidityPeriodInDaysMin" ), mValidityPeriodInDaysMin, 1 );
  addItem( itemValidityPeriodInDaysMin, QStringLiteral( "ValidityPeriodInDaysMin" ) );
  KConfigSkeleton::ItemInt  *itemValidityPeriodInDaysMax;
  itemValidityPeriodInDaysMax = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "ValidityPeriodInDaysMax" ), mValidityPeriodInDaysMax, -1 );
  addItem( itemValidityPeriodInDaysMax, QStringLiteral( "ValidityPeriodInDaysMax" ) );
  KConfigSkeleton::ItemBool  *itemHideAdvanced;
  itemHideAdvanced = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "HideAdvanced" ), mHideAdvanced, false );
  addItem( itemHideAdvanced, QStringLiteral( "HideAdvanced" ) );

  setCurrentGroup( QStringLiteral( "Certification" ) );

  KConfigSkeleton::ItemInt  *itemCertificationValidityInDays;
  itemCertificationValidityInDays = new KConfigSkeleton::ItemInt( currentGroup(), QStringLiteral( "CertificationValidityInDays" ), mCertificationValidityInDays, 0 );
  addItem( itemCertificationValidityInDays, QStringLiteral( "CertificationValidityInDays" ) );

  setCurrentGroup( QStringLiteral( "ChecksumOperations" ) );

  KConfigSkeleton::ItemString  *itemChecksumDefinitionId;
  itemChecksumDefinitionId = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "checksum-definition-id" ), mChecksumDefinitionId, QStringLiteral( "sha256sum" ) );
  addItem( itemChecksumDefinitionId, QStringLiteral( "ChecksumDefinitionId" ) );

  setCurrentGroup( QStringLiteral( "Clipboard" ) );

  KConfigSkeleton::ItemBool  *itemShowResultsAfterSigningClipboard;
  itemShowResultsAfterSigningClipboard = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowResultsAfterSigning" ), mShowResultsAfterSigningClipboard, true );
  addItem( itemShowResultsAfterSigningClipboard, QStringLiteral( "ShowResultsAfterSigningClipboard" ) );
  KConfigSkeleton::ItemBool  *itemShowResultsAfterEncryptingClipboard;
  itemShowResultsAfterEncryptingClipboard = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowResultsAfterEncryption" ), mShowResultsAfterEncryptingClipboard, true );
  addItem( itemShowResultsAfterEncryptingClipboard, QStringLiteral( "ShowResultsAfterEncryptingClipboard" ) );

  setCurrentGroup( QStringLiteral( "CMS" ) );

  KConfigSkeleton::ItemBool  *itemCmsEnabled;
  itemCmsEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "Enabled" ), mCmsEnabled, true );
  addItem( itemCmsEnabled, QStringLiteral( "cmsEnabled" ) );
  KConfigSkeleton::ItemBool  *itemCmsCertificateCreationAllowed;
  itemCmsCertificateCreationAllowed = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AllowCertificateCreation" ), mCmsCertificateCreationAllowed, true );
  addItem( itemCmsCertificateCreationAllowed, QStringLiteral( "cmsCertificateCreationAllowed" ) );
  KConfigSkeleton::ItemBool  *itemSaveCSRAsPEM;
  itemSaveCSRAsPEM = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "SaveCSRAsPEM" ), mSaveCSRAsPEM, false );
  addItem( itemSaveCSRAsPEM, QStringLiteral( "SaveCSRAsPEM" ) );

  setCurrentGroup( QStringLiteral( "ConfigurationDialog" ) );

  KConfigSkeleton::ItemBool  *itemShowAppearanceConfiguration;
  itemShowAppearanceConfiguration = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowAppearanceConfiguration" ), mShowAppearanceConfiguration, true );
  addItem( itemShowAppearanceConfiguration, QStringLiteral( "ShowAppearanceConfiguration" ) );
  KConfigSkeleton::ItemBool  *itemShowCryptoOperationsConfiguration;
  itemShowCryptoOperationsConfiguration = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowCryptoOperationsConfiguration" ), mShowCryptoOperationsConfiguration, true );
  addItem( itemShowCryptoOperationsConfiguration, QStringLiteral( "ShowCryptoOperationsConfiguration" ) );
  KConfigSkeleton::ItemBool  *itemShowDirectoryServicesConfiguration;
  itemShowDirectoryServicesConfiguration = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowDirectoryServicesConfiguration" ), mShowDirectoryServicesConfiguration, true );
  addItem( itemShowDirectoryServicesConfiguration, QStringLiteral( "ShowDirectoryServicesConfiguration" ) );
  KConfigSkeleton::ItemBool  *itemShowGnuPGSystemConfiguration;
  itemShowGnuPGSystemConfiguration = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowGnuPGSystemConfiguration" ), mShowGnuPGSystemConfiguration, true );
  addItem( itemShowGnuPGSystemConfiguration, QStringLiteral( "ShowGnuPGSystemConfiguration" ) );
  KConfigSkeleton::ItemBool  *itemShowSmartCardsConfiguration;
  itemShowSmartCardsConfiguration = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowSmartCardsConfiguration" ), mShowSmartCardsConfiguration, true );
  addItem( itemShowSmartCardsConfiguration, QStringLiteral( "ShowSmartCardsConfiguration" ) );
  KConfigSkeleton::ItemBool  *itemShowSMimeValidationConfiguration;
  itemShowSMimeValidationConfiguration = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowSMimeValidationConfiguration" ), mShowSMimeValidationConfiguration, true );
  addItem( itemShowSMimeValidationConfiguration, QStringLiteral( "ShowSMimeValidationConfiguration" ) );

  setCurrentGroup( QStringLiteral( "DN" ) );

  KConfigSkeleton::ItemStringList  *itemAttributeOrder;
  itemAttributeOrder = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "AttributeOrder" ), mAttributeOrder );
  addItem( itemAttributeOrder, QStringLiteral( "AttributeOrder" ) );

  setCurrentGroup( QStringLiteral( "Export" ) );

  KConfigSkeleton::ItemBool  *itemAllowPublicKeyUpload;
  itemAllowPublicKeyUpload = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AllowPublicKeyUpload" ), mAllowPublicKeyUpload, true );
  addItem( itemAllowPublicKeyUpload, QStringLiteral( "AllowPublicKeyUpload" ) );

  setCurrentGroup( QStringLiteral( "FileOperations" ) );

  KConfigSkeleton::ItemBool  *itemUsePGPFileExt;
  itemUsePGPFileExt = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "file-extension-pgp" ), mUsePGPFileExt, false );
  addItem( itemUsePGPFileExt, QStringLiteral( "UsePGPFileExt" ) );
  KConfigSkeleton::ItemBool  *itemAutoExtractArchives;
  itemAutoExtractArchives = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "auto-extract-archives" ), mAutoExtractArchives, true );
  addItem( itemAutoExtractArchives, QStringLiteral( "AutoExtractArchives" ) );
  KConfigSkeleton::ItemString  *itemArchiveCommand;
  itemArchiveCommand = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "default-archive-cmd" ), mArchiveCommand, QStringLiteral( "tar" ) );
  addItem( itemArchiveCommand, QStringLiteral( "ArchiveCommand" ) );
  KConfigSkeleton::ItemBool  *itemAddASCIIArmor;
  itemAddASCIIArmor = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ascii-armor" ), mAddASCIIArmor, false );
  addItem( itemAddASCIIArmor, QStringLiteral( "AddASCIIArmor" ) );
  KConfigSkeleton::ItemBool  *itemDontUseTmpDir;
  itemDontUseTmpDir = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "dont-use-tmp-dir" ), mDontUseTmpDir, false );
  addItem( itemDontUseTmpDir, QStringLiteral( "DontUseTmpDir" ) );
  KConfigSkeleton::ItemBool  *itemSymmetricEncryptionOnly;
  itemSymmetricEncryptionOnly = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "symmetric-encryption-only" ), mSymmetricEncryptionOnly, false );
  addItem( itemSymmetricEncryptionOnly, QStringLiteral( "SymmetricEncryptionOnly" ) );
  KConfigSkeleton::ItemBool  *itemPublicKeyEncryptionOnly;
  itemPublicKeyEncryptionOnly = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "public-key-encryption-only" ), mPublicKeyEncryptionOnly, false );
  addItem( itemPublicKeyEncryptionOnly, QStringLiteral( "PublicKeyEncryptionOnly" ) );

  setCurrentGroup( QStringLiteral( "Groups" ) );

  KConfigSkeleton::ItemBool  *itemGroupsEnabled;
  itemGroupsEnabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "GroupsEnabled" ), mGroupsEnabled, true );
  addItem( itemGroupsEnabled, QStringLiteral( "GroupsEnabled" ) );

  setCurrentGroup( QStringLiteral( "Import" ) );

  KConfigSkeleton::ItemBool  *itemRetrieveSignerKeysAfterImport;
  itemRetrieveSignerKeysAfterImport = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "RetrieveSignerKeysAfterImport" ), mRetrieveSignerKeysAfterImport, false );
  addItem( itemRetrieveSignerKeysAfterImport, QStringLiteral( "RetrieveSignerKeysAfterImport" ) );
  KConfigSkeleton::ItemBool  *itemQueryWKDsForAllUserIDs;
  itemQueryWKDsForAllUserIDs = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "QueryWKDsForAllUserIDs" ), mQueryWKDsForAllUserIDs, false );
  addItem( itemQueryWKDsForAllUserIDs, QStringLiteral( "QueryWKDsForAllUserIDs" ) );

  setCurrentGroup( QStringLiteral( "Notifications" ) );

  KConfigSkeleton::ItemBool  *itemShowExpiryNotifications;
  itemShowExpiryNotifications = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowExpiryNotifications" ), mShowExpiryNotifications, true );
  addItem( itemShowExpiryNotifications, QStringLiteral( "ShowExpiryNotifications" ) );

  setCurrentGroup( QStringLiteral( "Privacy" ) );

  KConfigSkeleton::ItemStringList  *itemBlockedUrlSchemes;
  itemBlockedUrlSchemes = new KConfigSkeleton::ItemStringList( currentGroup(), QStringLiteral( "BlockedUrlSchemes" ), mBlockedUrlSchemes );
  addItem( itemBlockedUrlSchemes, QStringLiteral( "BlockedUrlSchemes" ) );

  setCurrentGroup( QStringLiteral( "Smartcard" ) );

  KConfigSkeleton::ItemBool  *itemAlwaysSearchCardOnKeyserver;
  itemAlwaysSearchCardOnKeyserver = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AlwaysSearchCardOnKeyserver" ), mAlwaysSearchCardOnKeyserver, false );
  addItem( itemAlwaysSearchCardOnKeyserver, QStringLiteral( "AlwaysSearchCardOnKeyserver" ) );
  KConfigSkeleton::ItemBool  *itemAutoLoadP15Certs;
  itemAutoLoadP15Certs = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AutoLoadP15Certs" ), mAutoLoadP15Certs, true );
  addItem( itemAutoLoadP15Certs, QStringLiteral( "autoLoadP15Certs" ) );

  setCurrentGroup( QStringLiteral( "SMime Validation" ) );

  KConfigSkeleton::ItemUInt  *itemRefreshInterval;
  itemRefreshInterval = new KConfigSkeleton::ItemUInt( currentGroup(), QStringLiteral( "refresh-interval" ), mRefreshInterval, 1 );
  itemRefreshInterval->setMaxValue(24);
  addItem( itemRefreshInterval, QStringLiteral( "RefreshInterval" ) );

  setCurrentGroup( QStringLiteral( "General" ) );

  KConfigSkeleton::ItemBool  *itemProfilesDisabled;
  itemProfilesDisabled = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ProfilesDisabled" ), mProfilesDisabled, false );
  addItem( itemProfilesDisabled, QStringLiteral( "ProfilesDisabled" ) );
  KConfigSkeleton::ItemBool  *itemAllowCustomDebugCommands;
  itemAllowCustomDebugCommands = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "AllowCustomDebugCommands" ), mAllowCustomDebugCommands, true );
  addItem( itemAllowCustomDebugCommands, QStringLiteral( "AllowCustomDebugCommands" ) );
  KConfigSkeleton::ItemBool  *itemShowResultsAfterSignEncrypt;
  itemShowResultsAfterSignEncrypt = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "ShowResultsAfterSignEncrypt" ), mShowResultsAfterSignEncrypt, true );
  addItem( itemShowResultsAfterSignEncrypt, QStringLiteral( "ShowResultsAfterSignEncrypt" ) );
  KConfigSkeleton::ItemBool  *itemRestoreSignEncryptValues;
  itemRestoreSignEncryptValues = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "RestoreSignEncryptValues" ), mRestoreSignEncryptValues, true );
  addItem( itemRestoreSignEncryptValues, QStringLiteral( "RestoreSignEncryptValues" ) );

  setCurrentGroup( QStringLiteral( "Tooltip" ) );

  KConfigSkeleton::ItemBool  *itemShowValidity;
  itemShowValidity = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "show-validity" ), mShowValidity, true );
  addItem( itemShowValidity, QStringLiteral( "ShowValidity" ) );
  KConfigSkeleton::ItemBool  *itemShowOwnerInformation;
  itemShowOwnerInformation = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "show-owner-information" ), mShowOwnerInformation, false );
  addItem( itemShowOwnerInformation, QStringLiteral( "ShowOwnerInformation" ) );
  KConfigSkeleton::ItemBool  *itemShowCertificateDetails;
  itemShowCertificateDetails = new KConfigSkeleton::ItemBool( currentGroup(), QStringLiteral( "show-certificate-details" ), mShowCertificateDetails, false );
  addItem( itemShowCertificateDetails, QStringLiteral( "ShowCertificateDetails" ) );

  setCurrentGroup( QStringLiteral( "Welcome" ) );

  KConfigSkeleton::ItemString  *itemWelcomeText;
  itemWelcomeText = new KConfigSkeleton::ItemString( currentGroup(), QStringLiteral( "welcome-text" ), mWelcomeText );
  addItem( itemWelcomeText, QStringLiteral( "WelcomeText" ) );
}

Settings::~Settings()
{
}

