/********************************************************************************
** Form generated from reading UI file 'signingcertificateselectionwidget.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SIGNINGCERTIFICATESELECTIONWIDGET_H
#define UI_SIGNINGCERTIFICATESELECTIONWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_SigningCertificateSelectionWidget
{
public:
    QGridLayout *gridLayout;
    QComboBox *pgpCombo;
    QComboBox *cmsCombo;
    QCheckBox *rememberCO;
    QSpacerItem *spacerItem;
    QLabel *pgpLabel;
    QLabel *cmsLabel;

    void setupUi(QWidget *SigningCertificateSelectionWidget)
    {
        if (SigningCertificateSelectionWidget->objectName().isEmpty())
            SigningCertificateSelectionWidget->setObjectName("SigningCertificateSelectionWidget");
        SigningCertificateSelectionWidget->resize(475, 178);
        gridLayout = new QGridLayout(SigningCertificateSelectionWidget);
        gridLayout->setObjectName("gridLayout");
        pgpCombo = new QComboBox(SigningCertificateSelectionWidget);
        pgpCombo->setObjectName("pgpCombo");
        pgpCombo->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLengthWithIcon);
        pgpCombo->setMinimumContentsLength(40);

        gridLayout->addWidget(pgpCombo, 1, 0, 1, 2);

        cmsCombo = new QComboBox(SigningCertificateSelectionWidget);
        cmsCombo->setObjectName("cmsCombo");
        cmsCombo->setSizeAdjustPolicy(QComboBox::AdjustToMinimumContentsLengthWithIcon);
        cmsCombo->setMinimumContentsLength(40);

        gridLayout->addWidget(cmsCombo, 3, 0, 1, 2);

        rememberCO = new QCheckBox(SigningCertificateSelectionWidget);
        rememberCO->setObjectName("rememberCO");

        gridLayout->addWidget(rememberCO, 4, 0, 1, 2);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 5, 0, 1, 1);

        pgpLabel = new QLabel(SigningCertificateSelectionWidget);
        pgpLabel->setObjectName("pgpLabel");

        gridLayout->addWidget(pgpLabel, 0, 0, 1, 2);

        cmsLabel = new QLabel(SigningCertificateSelectionWidget);
        cmsLabel->setObjectName("cmsLabel");

        gridLayout->addWidget(cmsLabel, 2, 0, 1, 2);

#if QT_CONFIG(shortcut)
        pgpLabel->setBuddy(pgpCombo);
        cmsLabel->setBuddy(cmsCombo);
#endif // QT_CONFIG(shortcut)

        retranslateUi(SigningCertificateSelectionWidget);

        QMetaObject::connectSlotsByName(SigningCertificateSelectionWidget);
    } // setupUi

    void retranslateUi(QWidget *SigningCertificateSelectionWidget)
    {
        rememberCO->setText(tr2i18n("Remember these as default for future operations", nullptr));
        pgpLabel->setText(tr2i18n("OpenPGP Signing Certificate:", nullptr));
        cmsLabel->setText(tr2i18n("S/MIME Signing Certificate:", nullptr));
        (void)SigningCertificateSelectionWidget;
    } // retranslateUi

};

namespace Ui {
    class SigningCertificateSelectionWidget: public Ui_SigningCertificateSelectionWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // SIGNINGCERTIFICATESELECTIONWIDGET_H

