/****************************************************************************
** Meta object code from reading C++ file 'signencryptwidget.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../kleopatra-26.04.2/src/crypto/gui/signencryptwidget.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'signencryptwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN4Kleo17SignEncryptWidgetE_t {};
} // unnamed namespace

template <> constexpr inline auto Kleo::SignEncryptWidget::qt_create_metaobjectdata<qt_meta_tag_ZN4Kleo17SignEncryptWidgetE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Kleo::SignEncryptWidget",
        "operationChanged",
        "",
        "Operations",
        "op",
        "keysChanged",
        "updateOp",
        "recipientsChanged",
        "certificateSelectionRequested",
        "CertificateLineEdit*",
        "w",
        "ownCertificateSelectionRequested",
        "Kleo::Dialogs::CertificateSelectionDialog::Options",
        "options",
        "UserIDSelectionCombo*",
        "combo"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'operationChanged'
        QtMocHelpers::SignalData<void(Operations)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 4 },
        }}),
        // Signal 'keysChanged'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'updateOp'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'recipientsChanged'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'certificateSelectionRequested'
        QtMocHelpers::SlotData<void(CertificateLineEdit *)>(8, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 9, 10 },
        }}),
        // Slot 'ownCertificateSelectionRequested'
        QtMocHelpers::SlotData<void(Kleo::Dialogs::CertificateSelectionDialog::Options, UserIDSelectionCombo *)>(11, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 12, 13 }, { 0x80000000 | 14, 15 },
        }}),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<SignEncryptWidget, qt_meta_tag_ZN4Kleo17SignEncryptWidgetE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Kleo::SignEncryptWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Kleo17SignEncryptWidgetE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Kleo17SignEncryptWidgetE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN4Kleo17SignEncryptWidgetE_t>.metaTypes,
    nullptr
} };

void Kleo::SignEncryptWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<SignEncryptWidget *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->operationChanged((*reinterpret_cast<std::add_pointer_t<Operations>>(_a[1]))); break;
        case 1: _t->keysChanged(); break;
        case 2: _t->updateOp(); break;
        case 3: _t->recipientsChanged(); break;
        case 4: _t->certificateSelectionRequested((*reinterpret_cast<std::add_pointer_t<CertificateLineEdit*>>(_a[1]))); break;
        case 5: _t->ownCertificateSelectionRequested((*reinterpret_cast<std::add_pointer_t<Kleo::Dialogs::CertificateSelectionDialog::Options>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<UserIDSelectionCombo*>>(_a[2]))); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< UserIDSelectionCombo* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (SignEncryptWidget::*)(Operations )>(_a, &SignEncryptWidget::operationChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (SignEncryptWidget::*)()>(_a, &SignEncryptWidget::keysChanged, 1))
            return;
    }
}

const QMetaObject *Kleo::SignEncryptWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Kleo::SignEncryptWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN4Kleo17SignEncryptWidgetE_t>.strings))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int Kleo::SignEncryptWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 6)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 6;
    }
    return _id;
}

// SIGNAL 0
void Kleo::SignEncryptWidget::operationChanged(Operations _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1);
}

// SIGNAL 1
void Kleo::SignEncryptWidget::keysChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
